package cn.com.ty.lift.business.framework.conf;

import cn.com.ty.lift.business.framework.aspect.JudgeMethodInterceptor;
import cn.com.ty.lift.common.aliservice.aliyunoss.AliyunOSS;
import cn.com.ty.lift.common.export.ExportUtils;
import cn.com.ty.lift.common.sql.SqlAnalysisInterceptor;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.IllegalSQLInterceptor;
import com.baomidou.mybatisplus.extension.plugins.PerformanceInterceptor;
import com.baomidou.mybatisplus.extension.plugins.SqlExplainInterceptor;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateDeserializer;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateTimeDeserializer;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalTimeDeserializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateSerializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateTimeSerializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalTimeSerializer;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.aspectj.AspectJExpressionPointcut;
import org.springframework.aop.support.DefaultPointcutAdvisor;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.jackson.Jackson2ObjectMapperBuilderCustomizer;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import java.time.format.DateTimeFormatter;

/**
 * <p>
 *     配置bean注入
 * </p>
 * @author wcz
 * @since  2019/12/3
 */
@Slf4j
@Data
@Configuration
@ConfigurationProperties(prefix = "aliyun.oss")
public class SystemConfiguration {

    private String endpoint;

    private String bucketName;

    private String accessKeySecret;

    private String accessKeyId;

    @PostConstruct
    private void print(){
        log.info("Hint: the Configuration for Aliyun OSS Client.");
        log.info("============================================================");
        log.info("###| endpoint       : {}", endpoint);
        log.info("###| accessKeyId    : {}", accessKeyId);
        log.info("###| accessKeySecret: {}", accessKeySecret);
        log.info("###| bucketName     : {}", bucketName);
        log.info("============================================================");
    }
    /**
     * 创建aliyun OSS bean,每次使用完连接释放
     */
    public AliyunOSS build(){
        if(StrUtil.hasEmpty(endpoint,accessKeyId,accessKeySecret,bucketName)){
            return AliyunOSS.me();
        }
        return AliyunOSS.me(endpoint,accessKeyId,accessKeySecret,bucketName);
    }

    /**
     * 全局格式化日期
     */
    @Bean
    public Jackson2ObjectMapperBuilderCustomizer jsonCustomizer() {
        return builder -> {
            builder.serializers(new LocalDateTimeSerializer(DateTimeFormatter.ofPattern(DatePattern.NORM_DATETIME_PATTERN)));
            builder.serializers(new LocalDateSerializer(DateTimeFormatter.ofPattern(DatePattern.NORM_DATE_PATTERN)));
            builder.serializers(new LocalTimeSerializer(DateTimeFormatter.ofPattern(DatePattern.NORM_TIME_PATTERN)));

            builder.deserializers(new LocalDateDeserializer(DateTimeFormatter.ofPattern(DatePattern.NORM_DATE_PATTERN)));
            builder.deserializers(new LocalDateTimeDeserializer(DateTimeFormatter.ofPattern(DatePattern.NORM_DATETIME_PATTERN)));
            builder.deserializers(new LocalTimeDeserializer(DateTimeFormatter.ofPattern(DatePattern.NORM_TIME_PATTERN)));

            builder.serializerByType(Long.class, ToStringSerializer.instance);
        };
    }

    /**
     * mybatis-plus 执行sql性能监测
     */
    @Bean
    @ConditionalOnMissingBean(PerformanceInterceptor.class)
    public SqlAnalysisInterceptor sqlAnalysisInterceptor(){
        SqlAnalysisInterceptor sqlAnalysisInterceptor = new SqlAnalysisInterceptor();
        //格式化执行的sql
        sqlAnalysisInterceptor.setFormat(true);
        //sql写入日志文件
        sqlAnalysisInterceptor.setLogWrite(true);
        return sqlAnalysisInterceptor;
    }

    /**
     * SQL是影响系统性能最重要的因素，所以拦截掉垃圾SQL语句
     */
//    @Bean
//    @ConditionalOnMissingBean
    public IllegalSQLInterceptor illegalSQLInterceptor(){
        IllegalSQLInterceptor illegalSQLInterceptor = new IllegalSQLInterceptor();
        return illegalSQLInterceptor;
    }

    /**
     * 防止全表更新与删除
     */
//    @Bean
//    @ConditionalOnMissingBean
    public SqlExplainInterceptor sqlExplainInterceptor(){
        SqlExplainInterceptor sqlExplainInterceptor = new SqlExplainInterceptor();
        return sqlExplainInterceptor;
    }

    @Bean
    public ExportUtils exportUtils(){
        return new ExportUtils();
    }
    /**
     * 方法参数校验拦截器,标注了@Judger的方法，会检查参数，如果参数不满足条件，直接抛出异常
     */
    @Bean
    public DefaultPointcutAdvisor defaultPointcutAdvisor(){
        AspectJExpressionPointcut pointcut = new AspectJExpressionPointcut();
        pointcut.setExpression("@annotation(cn.com.ty.lift.common.judge.Judger)");
        // 配置增强类advisor
        DefaultPointcutAdvisor advisor = new DefaultPointcutAdvisor();
        advisor.setPointcut(pointcut);
        advisor.setAdvice(new JudgeMethodInterceptor());
        advisor.setOrder(2);
        return advisor;
    }
}
