package cn.com.ty.lift.system.config;

import cn.com.ty.lift.common.judge.Judgement;
import cn.com.ty.lift.common.judge.Judger;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.web.bind.annotation.RequestBody;

import java.lang.reflect.Method;
import java.lang.reflect.Parameter;

/**
 * the {@link MethodInterceptor} to judge the parameters
 *
 * @author wcz
 * @since 2020/2/25
 */
public class JudgeMethodInterceptor implements MethodInterceptor {
    @Override
    public Object invoke(MethodInvocation invocation) throws Throwable {
        //get the method
        Method method = invocation.getMethod();
        //check whether the method is present with judger annotation.
        if (!method.isAnnotationPresent(Judger.class)) {
            return invocation.proceed();
        }

        Object[] arguments = invocation.getArguments();
        //just return if there is not any argument.
        if (arguments.length == 0) {
            return invocation.proceed();
        }
        Object object = null;
        //by default, judge the first arg present with RequestBody, otherwise the first in the arg list.
        Parameter[] parameters = method.getParameters();
        for (int i = 0; i < parameters.length; i++) {
            if(parameters[i].isAnnotationPresent(RequestBody.class)){
                object = arguments[i];
                break;
            }else{
                object = arguments[0];
            }
        }
        if (null == object) {
            return invocation.proceed();
        }
        //get the judger
        Judger judger = method.getDeclaredAnnotation(Judger.class);

        Judgement.INSTANCE.action(object, judger.fields());

        return invocation.proceed();
    }
}
